@@ -1,7 +1,7 @@
 """
 The Penultimate Rotamer Library
 Simon C. Lovell, J. Michael Word, Jane S. Richardson, and David C. Richardson
-PROTEINS: Structure, Function, and Genetics 40:389–408 (2000)
+PROTEINS: Structure, Function, and Genetics 40:389-408 (2000)
 """
 import json
 import os
@@ -10,3 +10,6 @@ import os
 basedir = os.path.dirname(os.path.abspath(__file__))
 with open(os.path.join(basedir, 'penultimate_lib.json')) as j:
     PENULTIMATE_2000 = json.load(j)
+
+with open(os.path.join(basedir, 'rotamer_bounds_lib.json')) as j:
+    ROTAMER_BOUNDS = json.load(j)
\ No newline at end of file
